// If rust-analyzer says it can't find `test`, you need to disable `allTargets` in it's
// configuration. This project has a `.cargo/config.toml` file that explicitly sets targets.
// See https://github.com/rust-lang/rust-analyzer/issues/3297
#![no_std]
#![no_main]

#[cfg(not(all(target_os = "none", target_pointer_width = "32", target_arch = "arm")))]
compile_error!("This program can only be compiled for arm32 bare metal");

use defmt::{debug, info};
use panic_probe as _;

use defmt_rtt as _; // global logger

use cortex_m_rt::entry;
use hal::{
    pac::{self},
    prelude::*,
    rcc::Config,
};

mod bench;
mod error_handlers; // defines a handler for evil hardfaults

#[entry]
fn main() -> ! {
    let dp = pac::Peripherals::take().unwrap();
    let mut cp = cortex_m::Peripherals::take().unwrap();

    // Configure the clock.
    let mut rcc = dp.RCC.freeze(Config::hsi16());

    // Acquire the GPIOA peripheral. This also enables the clock for GPIOA in
    // the RCC register.
    let gpioa = dp.GPIOA.split(&mut rcc);

    // Configure PA5 as output.
    let mut led = gpioa.pa5.into_push_pull_output().downgrade();

    // setup timer for measuring time of benchmarks
    cp.SYST.set_reload(0x00ffffff); // max
    cp.SYST.clear_current();
    cp.SYST.enable_counter();

    let mut delay = cp.SYST.delay(rcc.clocks);

    debug!("Asserting that libalgorithms is correctly loaded");
    assert!(algorithms::ffi_is_loaded(), "ffi is not loaded?");

    info!("Starting the Rust vs C benchmark");
    bench::bench(&mut led, &mut delay);
    info!("Benchmark done, nothing else to do");

    led.set_high().unwrap();
    #[allow(clippy::empty_loop)]
    loop {
        delay.delay_ms(200u32);
    }
}
